@@ -1,15 +1,28 @@
+from __future__ import annotations
+
 import copy
 import logging
+from typing import TYPE_CHECKING, Any
 
 import numpy as np
 from scipy.interpolate import interp1d
 
+if TYPE_CHECKING:
+    from numpy import ndarray
 
-def compute_proportion_overlap(a1, a2, b1, b2):
+    from clarity.enhancer.gha.audiogram import Audiogram
+
+
+def compute_proportion_overlap(
+    a1: float,
+    a2: float,
+    b1: float,
+    b2: float,
+) -> float:
     """Compute proportion of overlap of two ranges.
 
     For ranges (a1, a2) and (b1, b2), express the extent of the overlap
-    in the range as a proportion of the extent of range (b1, b2).add()
+    in the range as a proportion of the extent of range (b1, b2)
 
     e.g (4, 9) and (6, 15) -> overlap (6,9), so proportion is (9-6)/(15-6)
 
@@ -17,13 +30,12 @@ def compute_proportion_overlap(a1, a2, b1, b2):
     left = max(a1, b1)
     right = min(a2, b2)
     if left > right:
-        overlap = 0.0
-    else:
-        overlap = float(right - left) / (b2 - b1)
-    return overlap
+        return 0.0
+
+    return float(right - left) / (b2 - b1)
 
 
-def isothr(vsDesF):
+def isothr(vsDesF: list[int] | ndarray) -> ndarray:
     """Calculate conversion factors of HL thresholds to SPL thresholds.
 
     Translation of OpenMHA isothr.m. Calculates conversion factors of HL
@@ -128,7 +140,7 @@ def isothr(vsDesF):
     return vIsoThrDB
 
 
-def freq_interp_sh(f_in, y_in, f):
+def freq_interp_sh(f_in: ndarray, y_in: ndarray, f: list[float] | ndarray) -> ndarray:
     """Linear interpolation on logarithmic frequency scale.
 
     Has samples and hold on edges.
@@ -162,7 +174,12 @@ def freq_interp_sh(f_in, y_in, f):
     return y
 
 
-def gains(compr_thr_inputs, compr_thr_gains, compression_ratios, levels):
+def gains(
+    compr_thr_inputs: list[int] | ndarray,
+    compr_thr_gains: list[int] | ndarray,
+    compression_ratios: list[float] | ndarray,
+    levels: list[float] | ndarray,
+) -> ndarray:
     """Based on OpenMHA gains subfunction of gainrule_camfit_compr.
 
     Args:
@@ -191,8 +208,12 @@ def gains(compr_thr_inputs, compr_thr_gains, compression_ratios, levels):
 
 
 def gainrule_camfit_linear(
-    audiogram, sFitmodel, noisegatelevels=45, noisegateslope=1, max_output_level=100
-):
+    audiogram: Audiogram,
+    sFitmodel: dict[str, Any],
+    noisegatelevels: int | list[int] = 45,
+    noisegateslope: int = 1,
+    max_output_level: int = 100,
+) -> dict[str, ndarray]:
     """Apply linear Cambridge rule for hearing aid fittings 'CAMFIT'.
 
     Based on OpenMHA gainrule_camfit_linear.m. Applies linear Cambridge rule for
@@ -237,7 +258,7 @@ def gainrule_camfit_linear(
             intercepts = freq_interp_sh(
                 intercept_frequencies, intercepts, sFitmodel["frequencies"][0][0]
             )
-    except Exception:  # noqa E722
+    except TypeError:  # noqa E722
         intercepts = freq_interp_sh(
             intercept_frequencies, intercepts, sFitmodel["frequencies"]
         )
@@ -305,13 +326,13 @@ def gainrule_camfit_linear(
 
 
 def gainrule_camfit_compr(
-    audiogram,
-    sFitmodel,
-    noisegatelevels=45,
-    noisegateslope=1,
-    level=0,
-    max_output_level=100,
-):
+    audiogram: Audiogram,
+    sFitmodel: dict[str, Any],
+    noisegatelevels: int | list[int] = 45,
+    noisegateslope: int = 1,
+    level: int = 0,
+    max_output_level: int = 100,
+) -> dict[str, ndarray]:
     """Applies compressive Cambridge rule for hearing aid fittings 'CAMFIT'.
 
     Translation of OpenMHA gainrule_camfit_compr.m.
@@ -378,7 +399,7 @@ def gainrule_camfit_compr(
             16000,
         ]
     )
-    LTASS_edge_freq = np.zeros((26))
+    LTASS_edge_freq = np.zeros(26)
     LTASS_edge_freq[1:-1] = np.sqrt(LTASS_freq[0:-1] * LTASS_freq[1:])
     LTASS_edge_freq[-1] = 16000 * np.power(2, (1 / 6))
     LTASS_lev = np.array(
@@ -456,7 +477,7 @@ def gainrule_camfit_compr(
         audiogram, sFitmodel, noisegatelevels, noisegateslope, max_output_level
     )
     insertion_gains = Gmid["insertion_gains"]
-    Gmid = Gmid["sGt"]
+    gmid_sgt = Gmid["sGt"]
 
     # Calculate compression ratios
     compression_ratio = np.zeros((np.size(frequencies), 2))
@@ -471,7 +492,7 @@ def gainrule_camfit_compr(
 
     for i, levels in enumerate([audiogram.levels_l, audiogram.levels_r]):
         if level != 0:
-            tmp = Lmid + Gmid[cr_idx, :, i].flatten() - Lmin - Gmin[:, i]
+            tmp = Lmid + gmid_sgt[cr_idx, :, i].flatten() - Lmin - Gmin[:, i]
         else:
             tmp = Lmid + insertion_gains[:, i] - Lmin - Gmin[:, i]
         idx = [i for i, x in enumerate(tmp < 13) if x]
