@@ -1,14 +1,16 @@
 """Support for the MSBG hearing loss model."""
+from __future__ import annotations
+
 import json
 import logging
 import math
-import os
-from typing import Dict, Optional, Tuple, Union
+from pathlib import Path
 
 import numpy as np
 import scipy
 import scipy.signal
 import soundfile
+from numpy import float64, ndarray
 from soundfile import SoundFile
 
 # measure rms parameters
@@ -16,7 +18,6 @@ WIN_SECS = 0.01
 
 # read & write signal parameters
 MSBG_FS = 44100
-TEST_NBITS = 16
 
 # fmt: off
 HZ = np.array(
@@ -95,7 +96,7 @@ EMPHASIS = np.array(
 ) * (7.5 / 9)
 
 
-def read_gtf_file(gtf_file: str) -> Dict:
+def read_gtf_file(gtf_file: str) -> dict[str, str | ndarray | int | float]:
     """Read a gammatone filterbank file.
 
     List data is converted into numpy arrays.
@@ -103,9 +104,8 @@ def read_gtf_file(gtf_file: str) -> Dict:
     """
 
     # Fix filename if necessary
-    dirname = os.path.dirname(os.path.abspath(__file__))
-    gtf_file = os.path.join(dirname, gtf_file)
-    with open(gtf_file, "r", encoding="utf-8") as fp:
+    gtf_file_path = Path(__file__).parent / gtf_file
+    with gtf_file_path.open("r", encoding="utf-8") as fp:
         data = json.load(fp)
     for key in data:
         if isinstance(data[key], list):
@@ -115,21 +115,22 @@ def read_gtf_file(gtf_file: str) -> Dict:
 
 def firwin2(
     n_taps: int,
-    frequencies: np.ndarray,
-    filter_gains: np.ndarray,
-    window: Union[None, str, tuple] = None,
-    antisymmetric: Optional[bool] = None,
-):  # pylint: disable=W0613
+    frequencies: list[float] | ndarray,
+    filter_gains: list[int | float] | ndarray,
+    window: tuple[str, int] | str | None = None,
+    antisymmetric: bool | None = None,  # pylint: disable=W0613
+) -> ndarray:  # pylint: disable=W0613
     """FIR filter design using the window method.
 
     Partial implementation of scipy firwin2 but using our own MATLAB-derived fir2.
 
     Args:
         n_taps (int): The number of taps in the FIR filter.
-        frequencies (ndarray): The frequency sampling points. 0.0 to 1.0 with 1.0 being Nyquist.
+        frequencies (ndarray): The frequency sampling points. 0.0 to 1.0 with 1.0
+            being Nyquist.
         filter_gains (ndarray): The filter gains at the frequency sampling points.
-        window (string or (string, float), optional): See scipy.firwin2 (default: (None))
-        antisymmetric (bool, optional): Unused but present to main compatability
+        window (string or (string, float), optional): See scipy.firwin2. Default is None
+        antisymmetric (bool, optional): Unused but present to maintain compatibility
             with scipy firwin2.
 
     Returns:
@@ -158,18 +159,18 @@ def firwin2(
 
 def fir2(
     filter_length: int,
-    frequencies: np.ndarray,
-    filter_gains: np.ndarray,
-    n_interpolate: Optional[int] = None,
-) -> Tuple[np.ndarray, int]:
+    frequencies: list[float] | ndarray,
+    filter_gains: list[int | float] | ndarray,
+    n_interpolate: ndarray | None = None,
+) -> tuple[ndarray, int]:
     """FIR arbitrary shape filter design using the frequency sampling method.
 
     Translation of MATLAB fir2.
 
     Args:
         filter_length (int): Order
-        frequencies (ndarray): The frequency sampling points (0 < frequencies < 1) where 1 is Nyquist rate.
-                        First and last elements must be 0 and 1 respectively.
+        frequencies (ndarray): The frequency sampling points (0 < frequencies < 1) where
+            1 is Nyquist rate. First and last elements must be 0 and 1 respectively.
         filter_gains (ndarray): The filter gains at the frequency sampling points.
         n_interpolate (int, optional): Number of points for freq response interpolation
             (default: max(smallest power of 2 greater than nn, 512))
@@ -214,7 +215,7 @@ def fir2(
             ne = int(np.fix(frequencies[i + 1] * n_interpolate)) - 1
 
         j = np.arange(nb, ne + 1)
-        inc: Union[float, np.ndarray] = 0.0 if nb == ne else (j - nb) / (ne - nb)
+        inc: float | np.ndarray = 0.0 if nb == ne else (j - nb) / (ne - nb)
         H[nb : (ne + 1)] = inc * filter_gains[i + 1] + (1 - inc) * filter_gains[i]
         nb = ne + 1
 
@@ -231,15 +232,19 @@ def fir2(
 
 
 def gen_tone(
-    freq: float, duration: float, sample_frequency: float = 44100.0, level: float = 0.0
-) -> np.ndarray:
+    freq: int,
+    duration: float,
+    sample_frequency: float = 44100.0,
+    level: int | float | float64 = 0.0,
+) -> ndarray:
     """Generate a pure tone.
 
     Args:
         freq (float): Frequency of tone in Hz.
         duration (float): Duration of tone in seconds.
-        sample_frequency (float, optional): Sample rate of generated tone in Hz. (default: 44100)
-        level (float, optional): Level of tone in dB SPL. (default: 0)
+        sample_frequency (float, optional): Sample rate of generated tone in Hz.
+            Default is 44100.
+        level (float, optional): Level of tone in dB SPL. Default is 0.
 
     Returns:
         np.ndarray
@@ -258,14 +263,15 @@ def gen_tone(
 
 
 def gen_eh2008_speech_noise(
-    duration: float,
+    duration: int | float,
     sample_frequency: float = 44100.0,
-    level: Optional[float] = None,
-    supplied_b: Optional[np.ndarray] = None,
-) -> np.ndarray:
+    level: int | float | float64 | None = None,
+    supplied_b: None = None,
+) -> ndarray:
     """Generate speech shaped noise.
 
-    Start with white noise and re-shape to ideal SII, ie flat to 500 Hz, and sloping -9db/oct beyond that.
+    Start with white noise and re-shape to ideal SII, ie flat to 500 Hz, and sloping
+        -9db/oct beyond that.
 
     Slightly different shape from SII stylised same as EarHEar 2008 paper, Moore et al.
 
@@ -273,7 +279,8 @@ def gen_eh2008_speech_noise(
         duration (float): Duration of signal in seconds
         sample_frequency (float): Sampling rate
         level (float, optional): Normalise to level dB if present
-        supplied_b (ndarray, optional): High-pass filter (default: uses built-in pre-emphasis filter)
+        supplied_b (ndarray, optional): High-pass filter. Default uses built-in
+            pre-emphasis filter
 
     Returns:
         ndarray: Noise signal
@@ -334,11 +341,11 @@ def gen_eh2008_speech_noise(
 
 
 def generate_key_percent(
-    signal: np.ndarray,
-    threshold_db: float,
+    signal: ndarray,
+    threshold_db: float64,
     window_length: int,
-    percent_to_track: Optional[float] = None,
-) -> Tuple[np.ndarray, float]:
+    percent_to_track: float | None = None,
+) -> tuple[ndarray, float64]:
     """Generate key percent.
     Locates frames above some energy threshold or tracks a certain percentage
     of frames. To track a certain percentage of frames in order to get measure
@@ -348,7 +355,8 @@ def generate_key_percent(
         signal (ndarray): The signal to analyse.
         threshold_db (float): fixed energy threshold (dB).
         window_length (int): length of window in samples.
-        percent_to_track (float, optional): Track a percentage of frames (default: {None}).
+        percent_to_track (float, optional): Track a percentage of frames.
+            Default is None
 
     Raises:
         ValueError: percent_to_track is set too high.
@@ -422,7 +430,8 @@ def generate_key_percent(
     # histogram should produce a two-peaked curve: thresh should be set in valley
     # between the two peaks, and set threshold a bit above that,
     # as it heads for main peak
-    # FixMe : Could Otsu's method (from image processing) be used here? https://en.wikipedia.org/wiki/Otsu's_method
+    # TODO : Could Otsu's method (from image processing) be used here?
+    # https://en.wikipedia.org/wiki/Otsu's_method
     frame_index = np.nonzero(every_db >= expected)[0]
     valid_frames = len(frame_index)
     key = np.zeros((1, valid_frames * window_length))[0]
@@ -440,11 +449,11 @@ def generate_key_percent(
 
 
 def measure_rms(
-    signal: np.ndarray,
-    sample_frequency: float,
-    db_rel_rms: float,
-    percent_to_track: Optional[float] = None,
-) -> tuple:
+    signal: ndarray,
+    sample_frequency: int,
+    db_rel_rms: int | float,
+    percent_to_track: float | None = None,
+) -> tuple[float64, ndarray, float64, float]:
     """Measure Root Mean Square.
 
     A sophisticated method of measuring RMS in a file. It splits the signal up into
@@ -490,29 +499,30 @@ def measure_rms(
     return rms, idx, rel_db_thresh, active
 
 
-def pad(signal, length):
+def pad(signal: ndarray, length: int) -> ndarray:
     """Zero pad signal to required length.
 
     Assumes required length is not less than input length.
     """
-    assert length >= signal.shape[0]
+    if length < signal.shape[0]:
+        raise ValueError("Required length is less than input length")
     return np.pad(
         signal, [(0, length - signal.shape[0])] + [(0, 0)] * (len(signal.shape) - 1)
     )
 
 
 def read_signal(
-    filename: str,
+    filename: str | Path,
     offset: int = 0,
     nsamples: int = -1,
     nchannels: int = 0,
     offset_is_samples: bool = False,
-) -> np.ndarray:
+) -> ndarray:
     """Read a wavefile and return as numpy array of floats.
 
     Args:
-        filename (string): Name of file to read
-        offset (int, optional): Offset in samples or seconds (from start). Defaults to 0.
+        filename (str|Path): Name of file to read
+        offset (int, optional): Offset in samples or seconds (from start). Default is 0.
         nsamples (int): Number of samples.
         nchannels (int): expected number of channel (default: 0 = any number OK)
         offset_is_samples (bool): measurement units for offset (default: False)
@@ -520,14 +530,11 @@ def read_signal(
     Returns:
         np.ndarray: audio signal
     """
-    try:
-        wave_file = SoundFile(filename)
-    except Exception as e:
-        # Ensure incorrect error (24 bit) is not generated
-        raise Exception(f"Unable to read {filename}.") from e
+
+    wave_file = SoundFile(filename)
 
     if nchannels not in (0, wave_file.channels):
-        raise Exception(
+        raise ValueError(
             f"Wav file ({filename}) was expected to have {nchannels} channels."
         )
 
@@ -548,35 +555,41 @@ def read_signal(
 
 
 def write_signal(
-    filename: str,
-    signal: np.ndarray,
-    sample_frequency: float,
+    filename: str | Path,
+    signal: ndarray,
+    sample_frequency: int,
     floating_point: bool = True,
+    strict: bool = False,
 ) -> None:
     """Write a signal as fixed or floating point wav file.
 
+    NB: setting 'strict' to True will raise error on overflow. This would be
+    a more natural default but it would break existing code that did not
+    check for overflow.
+
     Args:
-        filename (str): name of file in to write to.
+        filename (str|Path): name of file in to write to.
         signal (ndarray): signal to write.
         sample_frequency (float): sampling frequency.
         floating_point (bool): write as floating point else an ints (default: True).
+        strict (bool): raise error if signal out of range for int16 (default: False).
     """
 
     if sample_frequency != MSBG_FS:
         logging.warning(
-            f"Sampling rate mismatch: {filename} with sample frequency = {sample_frequency}."
+            f"Sampling rate mismatch: {filename} with "
+            f"sample frequency = {sample_frequency}."
         )
         # raise ValueError("Sampling rate mismatch")
 
     if floating_point is False:
-        if TEST_NBITS == 16:
-            subtype = "PCM_16"
-            # If signal is float and we want int16
-            signal *= 32768
-            signal = signal.astype(np.dtype("int16"))
-            assert np.max(signal) <= 32767 and np.min(signal) >= -32768
-        elif TEST_NBITS == 24:
-            subtype = "PCM_24"
+        subtype = "PCM_16"
+        # Signal is float and we want to convert to int16
+        # *NB* Not  *= in next line as we need to make a copy
+        signal = signal * 32768
+        if strict and (np.max(signal) > 32767 or np.min(signal) < -32768):
+            raise ValueError("Signal out of range -1.0 to 1.0")
+        signal = signal.astype(np.dtype("int16"))
     else:
         subtype = "FLOAT"
 
