@@ -1,17 +1,19 @@
 """
 An FIR-based torch implementation of approximated MSBG hearing loss model
 """
+from __future__ import annotations
+
 import json
-import os
+from pathlib import Path
 
 import numpy as np
 import torch
-import torch.nn as nn
 import torch.nn.functional as F
 import torchaudio
 from scipy.fftpack import fft
 from scipy.interpolate import interp1d
 from scipy.signal import ellip, firwin, firwin2, freqz
+from torch import nn
 
 from clarity.evaluator.msbg.msbg_utils import (
     DF_ED,
@@ -21,160 +23,47 @@ from clarity.evaluator.msbg.msbg_utils import (
     ITU_HZ,
     MIDEAR,
 )
+from clarity.evaluator.msbg.smearing import make_smear_mat3
 
 EPS = 1e-8
 # old msbg matlab
 # set RMS so that peak of output file so that no clipping occurs, set so that
 # equiv0dBfileSPL > 100dB for LOUD input files
-ref_RMSdB = -31.2
-calib_dBSPL = (
-    65  # what RMS of INPUT speech file translates to in real world (unweighted)
-)
-equiv0dBfileSPL = (
-    calib_dBSPL - ref_RMSdB
-)  # what 0dB file signal would translate to in dB SPL: constant for cochlear_simulate function
+REF_RMS_DB = -31.2
+
+# what RMS of INPUT speech file translates to in real world (unweighted)
+CALIB_DB_SPL = 65
+
+# what 0dB file signal would translate to in dB SPL:
+# constant for cochlea_simulate function
+EQUIV_0_DB_FILE_SPL = CALIB_DB_SPL - REF_RMS_DB
 
 # clarity msbg
-equiv0dBSPL = 100
-ahr = 20
-equiv_0dB_SPL = equiv0dBSPL + ahr
-
-
-def generate_key_percent(signal, thr_dB, winlen):
-    if winlen != np.floor(winlen):
-        winlen = np.int(np.floor(winlen))
-        print("\nGenerate_key_percent: \t Window length must be integer")
-
-    siglen = len(signal)
-    expected = thr_dB.copy()  # expected threshold
-    non_zero = 10.0 ** ((expected - 30) / 10)  # put floor into histogram distribution
-
-    nframes = 0
-    totframes = np.int(np.floor(siglen / winlen))
-    every_dB = np.zeros(totframes)
-
-    for ix in range(totframes):
-        start = ix * winlen
-        this_sum = np.sum(signal[start : start + winlen] ** 2)  # sum of squares
-        every_dB[nframes] = 10 * np.log10(non_zero + this_sum / winlen)
-        nframes += 1
-
-    used_thr_dB = expected.copy()
-
-    # histogram should produce a two-peaked curve: thresh should be set in valley
-    # between the two peaks, and set a bit above that, as it heads for main peak
-    frame_idx = np.where(every_dB >= expected)[0]
-    valid_frames = len(frame_idx)
-    key = np.zeros(valid_frames * winlen, dtype=np.int)
-
-    # convert frame numbers into indices for signal
-    for ix in range(valid_frames):
-        key[ix * winlen : ix * winlen + winlen] = np.arange(
-            frame_idx[ix] * winlen, frame_idx[ix] * winlen + winlen, dtype=np.int
-        )
-    return key, used_thr_dB
-
-
-def measure_rms(signal, sr, dB_rel_rms):
-    """Measures toatal power of all 10 msec frams that are above a user-specified threshold
-
-    Args:
-        signal: input signal
-        sr: sampling rate
-        dB_rel_rms: threshold relative to first-stage rms (if it is made of a 2*1 array, second value over rules.)
-                    only single value supported currently
-    Returns:
-        tuple: The percentage of frames that are required to be tracked for measuring RMS (useful when DR compression
-               changes histogram shape)
-    """
-    win_secs = 0.01
-    # first RMS is of all signal
-    first_stage_rms = np.sqrt(np.mean(signal**2))
-    # use this RMS to generate key threshold to more accurate RMS
-    key_thr_dB = np.max([20 * np.log10(first_stage_rms) + dB_rel_rms, -80])
-    key, used_thr_dB = generate_key_percent(
-        signal, key_thr_dB, np.int(np.round(win_secs * sr))
-    )
-    # active = 100.0 * len(key) / len(signal)
-    rms = np.sqrt(np.mean(signal[key] ** 2))
-    rel_dB_thresh = used_thr_dB - 20 * np.log10(rms)
-    return rms, key, rel_dB_thresh
-
-
-def makesmearmat3(rl, ru, sr):
-    fftsize = 512
-    nyquist = np.int(fftsize // 2)
-    fnor = audfilt(1, 1, nyquist, sr)
-    fwid = audfilt(rl, ru, nyquist, sr)
-    fnext = np.hstack([fnor, np.zeros([nyquist, nyquist // 2])])
-
-    for i in np.arange(nyquist // 2 + 1, nyquist + 1, dtype=np.int):
-        fnext[i - 1, nyquist : np.min([2 * i - 1, 3 * nyquist // 2])] = np.flip(
-            fnor[
-                i - 1, np.max([1, 2 * i - 3 * nyquist // 2]) - 1 : (2 * i - nyquist - 1)
-            ]
-        )
-    fsmear = np.linalg.lstsq(fnext, fwid)[
-        0
-    ]  # https://stackoverflow.com/questions/33559946/numpy-vs-mldivide-matlab-operator
-    fsmear = fsmear[:nyquist, :]
-
-    return fsmear
-
-
-def audfilt(rl, ru, size, sr):
-    """Calculate an auditory filter array
-
-    Args:
-        rl: broadening factor on the lower side
-        ru: broadening factor on the upper side
-        size:
-        sr:
-
-    Returns:
-        np.ndarray
-    """
-    aud_filter = np.zeros([size, size])
-    aud_filter[0, 0] = 1.0
-    aud_filter[0, 0] = aud_filter[0, 0] / ((rl + ru) / 2)
-
-    g = np.zeros(size)
-    for i in np.arange(2, size + 1, 1, dtype=np.int):
-        fhz = (i - 1) * sr / (2 * size)
-        erbhz = 24.7 * ((fhz * 0.00437) + 1)
-        pl = 4 * fhz / (erbhz * rl)
-        pu = 4 * fhz / (erbhz * ru)
-        j = np.arange(1, i, dtype=np.int)
-        g[j - 1] = np.abs((i - j) / (i - 1))
-        aud_filter[i - 1, j - 1] = (1 + (pl * g[j - 1])) * np.exp(-pl * g[j - 1])
-        j = np.arange(i, size + 1, dtype=np.int)
-        g[j - 1] = np.abs((i - j) / (i - 1))
-        aud_filter[i - 1, j - 1] = (1 + (pu * g[j - 1])) * np.exp(-pu * g[j - 1])
-        aud_filter[i - 1, :] = aud_filter[i - 1, :] / (erbhz * (rl + ru) / (2 * 24.7))
-    return aud_filter
+EQUIV_0_DB_SPL = 100
+AHR = 20
+EQUIV_0_DB_SPL = EQUIV_0_DB_SPL + AHR
 
 
 class MSBGHearingModel(nn.Module):
     def __init__(
         self,
-        audiogram,
-        audiometric,
-        sr=44100,
-        spl_cali=True,
-        src_posn="ff",
-        kernel_size=1025,
-        device=None,
-    ):
+        audiogram: list[int],
+        audiometric: list[int],
+        sr: int = 44100,
+        spl_cali: bool = True,
+        src_position: str = "ff",
+        kernel_size: int = 1025,
+        device: str | None = None,
+    ) -> None:
         super().__init__()
         self.sr = sr
         self.spl_cali = spl_cali
-        self.src_posn = src_posn
+        self.src_position = src_position
         self.kernel_size = kernel_size
         if device is None:
             self.device = "cuda" if torch.cuda.is_available() else "cpu"
         else:
             self.device = device
-
         # settings for audiogram
 
         audiogram = np.array(audiogram)
@@ -189,16 +78,16 @@ class MSBGHearingModel(nn.Module):
             np.array([0.125, 0.25, 0.5, 0.75, 1, 1.5, 2, 3, 4, 6, 8, 10, 12, 14, 16])
             * 1000
         )
-        interpf = interp1d(audiometric, audiogram)
-        audiogram = interpf(audiogram_cfs)
+        interp_f = interp1d(audiometric, audiogram)
+        audiogram = interp_f(audiogram_cfs)
 
         # settings for src_to_cochlea_filt
 
-        if src_posn == "ff":
+        if src_position == "ff":
             src_corrn = FF_ED
-        elif src_posn == "df":
+        elif src_position == "df":
             src_corrn = DF_ED
-        elif src_posn == "ITU":
+        elif src_position == "ITU":
             interf_itu = interp1d(ITU_HZ, ITU_ERP_DRP)
             src_corrn = interf_itu(HZ)
         nyquist = sr / 2
@@ -210,21 +99,25 @@ class MSBGHearingModel(nn.Module):
         corrn_used = np.append(corrn[ixf_useful], last_corrn)
         corrn_forward = 10 ** (0.05 * corrn_used)
         corrn_backward = 10 ** (0.05 * -1 * corrn_used)
-        n_wdw = np.int(2 * np.floor((sr / 16e3) * 368 / 2))
-        coch_filter_forward = firwin2(
+        n_wdw = int(2 * np.floor((sr / 16e3) * 368 / 2))
+        cochlea_filter_forward = firwin2(
             n_wdw + 1, hz_used / nyquist, corrn_forward, window=("kaiser", 4)
         )
-        coch_filter_backward = firwin2(
+        cochlea_filter_backward = firwin2(
             n_wdw + 1, hz_used / nyquist, corrn_backward, window=("kaiser", 4)
         )
-        self.coch_padding = len(coch_filter_forward) // 2
-        self.coch_filter_forward = (
-            torch.tensor(coch_filter_forward, dtype=torch.float32, device=self.device)
+        self.cochlea_padding = len(cochlea_filter_forward) // 2
+        self.cochlea_filter_forward = (
+            torch.tensor(
+                cochlea_filter_forward, dtype=torch.float32, device=self.device
+            )
             .unsqueeze(0)
             .unsqueeze(1)
         )
-        self.coch_filter_backward = (
-            torch.tensor(coch_filter_backward, dtype=torch.float32, device=self.device)
+        self.cochlea_filter_backward = (
+            torch.tensor(
+                cochlea_filter_backward, dtype=torch.float32, device=self.device
+            )
             .unsqueeze(0)
             .unsqueeze(1)
         )
@@ -244,44 +137,36 @@ class MSBGHearingModel(nn.Module):
         # impairment degree affects smearing simulation, and now recruitment,
         # (assuming we do not have too much SEVERE losses present)
 
-        current_dir = os.path.dirname(os.path.abspath(__file__))
-        gtf_dir = os.path.join(current_dir, "../evaluator/msbg/msbg_hparams")
+        current_dir = Path(__file__).parent
+        gtf_dir = current_dir / "../evaluator/msbg/msbg_hparams"
         if impaired_degree > 56:
             severe_not_moderate = 1
-            GT4Bankfile = os.path.join(
-                gtf_dir, "GT4FBank_Brd3.0E_Spaced2.3E_44100Fs.json"
-            )
+            gt4_bank_file = gtf_dir / "GT4FBank_Brd3.0E_Spaced2.3E_44100Fs.json"
             bw_broaden_coef = 3
         elif impaired_degree > 35:
             severe_not_moderate = 0
-            GT4Bankfile = os.path.join(
-                gtf_dir, "GT4FBank_Brd2.0E_Spaced1.5E_44100Fs.json"
-            )
+            gt4_bank_file = gtf_dir / "GT4FBank_Brd2.0E_Spaced1.5E_44100Fs.json"
             bw_broaden_coef = 2
         elif impaired_degree > 15:
             severe_not_moderate = -1
-            GT4Bankfile = os.path.join(
-                gtf_dir, "GT4FBank_Brd1.5E_Spaced1.1E_44100Fs.json"
-            )
+            gt4_bank_file = gtf_dir / "GT4FBank_Brd1.5E_Spaced1.1E_44100Fs.json"
             bw_broaden_coef = 1
         else:
             severe_not_moderate = -2
-            GT4Bankfile = os.path.join(
-                gtf_dir, "GT4FBank_Brd1.5E_Spaced1.1E_44100Fs.json"
-            )
+            gt4_bank_file = gtf_dir / "GT4FBank_Brd1.5E_Spaced1.1E_44100Fs.json"
             bw_broaden_coef = 1
-        # GT4Bank = loadmat(GT4Bankfile)
-        with open(GT4Bankfile, "r", encoding="utf-8") as fp:
-            GT4Bank = json.load(fp)
+        # gt4_bank = loadmat(gt4_bank_file)
+        with gt4_bank_file.open("r", encoding="utf-8") as fp:
+            gt4_bank = json.load(fp)
 
         if severe_not_moderate > 0:
-            fsmear = makesmearmat3(4, 2, sr)
+            f_smear = make_smear_mat3(4, 2, sr)
         elif severe_not_moderate == 0:
-            fsmear = makesmearmat3(2.4, 1.6, sr)
+            f_smear = make_smear_mat3(2.4, 1.6, sr)
         elif severe_not_moderate == -1:
-            fsmear = makesmearmat3(1.6, 1.1, sr)
+            f_smear = make_smear_mat3(1.6, 1.1, sr)
         elif severe_not_moderate == -2:
-            fsmear = makesmearmat3(1.001, 1.001, sr)
+            f_smear = make_smear_mat3(1.001, 1.001, sr)
 
         self.smear_nfft = 512
         self.smear_win_len = 256
@@ -299,83 +184,85 @@ class MSBGHearingModel(nn.Module):
         self.smear_window = torch.tensor(
             smear_window, dtype=torch.float32, device=self.device
         )
-        self.fsmear = torch.tensor(fsmear, dtype=torch.float32, device=self.device)
+        self.f_smear = torch.tensor(f_smear, dtype=torch.float32, device=self.device)
 
         """ settings for recruitment"""
-        cf_expnsn = 0 * np.array(GT4Bank["GTn_CentFrq"])
-        eq_loud_db = 0 * np.array(GT4Bank["GTn_CentFrq"])
-        for ix_cfreq in range(len(GT4Bank["GTn_CentFrq"])):
-            if GT4Bank["GTn_CentFrq"][ix_cfreq] < audiogram_cfs[0]:
-                cf_expnsn[ix_cfreq] = catch_up / (catch_up - audiogram[0])
-            elif GT4Bank["GTn_CentFrq"][ix_cfreq] > audiogram_cfs[-1]:
-                cf_expnsn[ix_cfreq] = catch_up / (catch_up - audiogram[-1])
+        cf_expansion = 0 * np.array(gt4_bank["GTn_CentFrq"])
+        eq_loud_db = 0 * np.array(gt4_bank["GTn_CentFrq"])
+        for ix_cfreq in range(len(gt4_bank["GTn_CentFrq"])):
+            if gt4_bank["GTn_CentFrq"][ix_cfreq] < audiogram_cfs[0]:
+                cf_expansion[ix_cfreq] = catch_up / (catch_up - audiogram[0])
+            elif gt4_bank["GTn_CentFrq"][ix_cfreq] > audiogram_cfs[-1]:
+                cf_expansion[ix_cfreq] = catch_up / (catch_up - audiogram[-1])
             else:
-                interp_audiog = interp1d(audiogram_cfs, audiogram)
-                audiog_cf = interp_audiog(GT4Bank["GTn_CentFrq"][ix_cfreq])
-                cf_expnsn[ix_cfreq] = catch_up / (catch_up - audiog_cf)
+                interp_audiogram = interp1d(audiogram_cfs, audiogram)
+                audiog_cf = interp_audiogram(gt4_bank["GTn_CentFrq"][ix_cfreq])
+                cf_expansion[ix_cfreq] = catch_up / (catch_up - audiog_cf)
             eq_loud_db[ix_cfreq] = catch_up
 
-        self.nchans = GT4Bank["NChans"]
+        self.n_chans = gt4_bank["NChans"]
         self.gtn_denoms = torch.tensor(
-            GT4Bank["GTn_denoms"], dtype=torch.float32, device=self.device
+            gt4_bank["GTn_denoms"], dtype=torch.float32, device=self.device
         )
         self.gtn_nums = torch.tensor(
-            GT4Bank["GTn_nums"], dtype=torch.float32, device=self.device
+            gt4_bank["GTn_nums"], dtype=torch.float32, device=self.device
         )
         self.hp_denoms = torch.tensor(
-            GT4Bank["HP_denoms"], dtype=torch.float32, device=self.device
+            gt4_bank["HP_denoms"], dtype=torch.float32, device=self.device
         )
         self.hp_nums = torch.tensor(
-            GT4Bank["HP_nums"], dtype=torch.float32, device=self.device
+            gt4_bank["HP_nums"], dtype=torch.float32, device=self.device
         )
-        self.ngamma = int(GT4Bank["NGAMMA"])
-        self.gtn_delays = GT4Bank["GTnDelays"]
-        self.start2polehp = GT4Bank["Start2PoleHP"]
+        self.ngamma = int(gt4_bank["NGAMMA"])
+        self.gtn_delays = gt4_bank["GTnDelays"]
+        self.start_2_pole_hp = gt4_bank["Start2PoleHP"]
 
-        erbn_centfrq = GT4Bank["ERBn_CentFrq"]
-        chan_lpfB = []
-        chan_lpfA = []
+        erbn_centre_freq = gt4_bank["ERBn_CentFrq"]
+        chan_lpf_b = []
+        chan_lpf_a = []
         fir_lpf = []
-        for ixch in range(self.nchans):
-            fc_envlp = (30 / 40) * np.min([100, erbn_centfrq[ixch]])
-            chan_lpfB_ch, chan_lpfA_ch = ellip(2, 0.25, 35, fc_envlp / (self.sr / 2))
-            chan_lpfB.append(chan_lpfB_ch)
-            chan_lpfA.append(chan_lpfA_ch)
+        for ixch in range(self.n_chans):
+            fc_envelope = (30 / 40) * np.min([100, erbn_centre_freq[ixch]])
+            chan_lpf_b_ch, chan_lpf_a_ch = ellip(
+                2, 0.25, 35, fc_envelope / (self.sr / 2)
+            )
+            chan_lpf_b.append(chan_lpf_b_ch)
+            chan_lpf_a.append(chan_lpf_a_ch)
             fir_lpf_ch = firwin(
-                self.kernel_size, fc_envlp / (self.sr / 2), pass_zero="lowpass"
+                self.kernel_size, fc_envelope / (self.sr / 2), pass_zero="lowpass"
             ) / np.sqrt(
                 2
             )  # sqrt(2) is for the consistency with IIR
             fir_lpf.append(fir_lpf_ch)
-        self.chan_lpfB = torch.tensor(
-            np.array(chan_lpfB), dtype=torch.float32, device=self.device
+        self.chan_lpf_b = torch.tensor(
+            np.array(chan_lpf_b), dtype=torch.float32, device=self.device
         )
-        self.chan_lpfA = torch.tensor(
-            np.array(chan_lpfA), dtype=torch.float32, device=self.device
+        self.chan_lpf_a = torch.tensor(
+            np.array(chan_lpf_a), dtype=torch.float32, device=self.device
         )
         self.fir_lpf = torch.tensor(
             np.array(fir_lpf), dtype=torch.float32, device=self.device
         ).unsqueeze(1)
 
-        self.expnsn_m1 = torch.tensor(
-            cf_expnsn - 1, dtype=torch.float32, device=self.device
+        self.expansion_m1 = torch.tensor(
+            cf_expansion - 1, dtype=torch.float32, device=self.device
         )
         # self.envlp_max = torch.tensor(10 ** (0.05 * (eq_loud_db - equiv0dBfileSPL)),
         # dtype=torch.float32, device=self.device)
-        self.envlp_max = torch.tensor(
-            10 ** (0.05 * (eq_loud_db - equiv_0dB_SPL)),
+        self.envelope_max = torch.tensor(
+            10 ** (0.05 * (eq_loud_db - EQUIV_0_DB_SPL)),
             dtype=torch.float32,
             device=self.device,
         )
 
-        recombination_dB = GT4Bank["Recombination_dB"]
-        self.recruitmnet_out_coef = torch.tensor(
-            10 ** (-0.05 * recombination_dB), dtype=torch.float32, device=self.device
+        recombination_db = gt4_bank["Recombination_dB"]
+        self.recruitment_out_coef = torch.tensor(
+            10 ** (-0.05 * recombination_db), dtype=torch.float32, device=self.device
         )
 
         "settings for FIR Gammatone Filters"
-        gt_cfreq = np.array(GT4Bank["GTn_CentFrq"])
-        gt_bw = np.array(GT4Bank["ERBn_CentFrq"]) * 1.1019 * bw_broaden_coef
+        gt_cfreq = np.array(gt4_bank["GTn_CentFrq"])
+        gt_bw = np.array(gt4_bank["ERBn_CentFrq"]) * 1.1019 * bw_broaden_coef
 
         self.padding = (self.kernel_size - 1) // 2
         n_lin = torch.linspace(
@@ -410,20 +297,20 @@ class MSBGHearingModel(nn.Module):
         self.peaks = torch.argmax(gammatone, dim=1)  # for gammatone delay calibration
 
         gammatone_sin = kernel * carrier_sin
-        filters = (gammatone * window_).view(self.nchans, 1, self.kernel_size)
+        filters = (gammatone * window_).view(self.n_chans, 1, self.kernel_size)
         # To get the normalised amplitude
         filters = filters.squeeze(1).cpu().numpy()
-        fr_max = np.zeros(self.nchans)
-        for i in range(self.nchans):
+        fr_max = np.zeros(self.n_chans)
+        for i in range(self.n_chans):
             fr = np.abs(fft(filters[i]))
             fr_ = fr[: int(self.kernel_size / 2)]
             fr_max[i] = np.max(fr_)
         amp = torch.tensor(fr_max, dtype=torch.float32, device=self.device)
         gammatone = gammatone / amp.unsqueeze(1)
         gammatone_sin = gammatone_sin / amp.unsqueeze(1)
-        self.gt_fir = (gammatone * window_).view(self.nchans, 1, self.kernel_size)
+        self.gt_fir = (gammatone * window_).view(self.n_chans, 1, self.kernel_size)
         self.gt_fir_sin = (gammatone_sin * window_).view(
-            self.nchans, 1, self.kernel_size
+            self.n_chans, 1, self.kernel_size
         )
 
         "settings for spl calibration"
@@ -431,10 +318,21 @@ class MSBGHearingModel(nn.Module):
         self.db_relative_rms = -12
         self.win_len = int(self.sr * win_sec)
 
-    def measure_rms(self, wav):
+    def measure_rms(self, wav: torch.Tensor) -> torch.Tensor:
+        """Compute RMS level of a signal.
+
+        Measures total power of all 10 msec frames that are above a specified
+          threshold of db_relative_rms
+
+        Args:
+            wav: input signal
+
+        Returns:
+            RMS level in dB
+        """
         bs = wav.shape[0]
-        ave_rms = torch.sqrt(torch.mean(wav**2, dim=1) + EPS)
-        thr_db = 20 * torch.log10(ave_rms + EPS) + self.db_relative_rms
+        average_rms = torch.sqrt(torch.mean(wav**2, dim=1) + EPS)
+        threshold_db = 20 * torch.log10(average_rms + EPS) + self.db_relative_rms
 
         num_frames = wav.shape[1] // self.win_len
         wav_reshaped = torch.reshape(
@@ -444,7 +342,7 @@ class MSBGHearingModel(nn.Module):
 
         key_frames = (
             torch.where(
-                db_frames > thr_db.unsqueeze(1),
+                db_frames > threshold_db.unsqueeze(1),
                 torch.tensor(1, dtype=torch.float32, device=self.device),
                 torch.tensor(0, dtype=torch.float32, device=self.device),
             )
@@ -459,19 +357,23 @@ class MSBGHearingModel(nn.Module):
         )
         return key_rms.unsqueeze(1)
 
-    def calibrate_spl(self, x):
+    def calibrate_spl(self, x: torch.Tensor) -> torch.Tensor:
         if self.spl_cali:
-            levelreFS = 10 * torch.log10(torch.mean(x**2, dim=1, keepdim=True) + EPS)
-            leveldBSPL = equiv_0dB_SPL + levelreFS
+            level_re_fs = 10 * torch.log10(
+                torch.mean(x**2, dim=1, keepdim=True) + EPS
+            )
+            level_db_spl = EQUIV_0_DB_SPL + level_re_fs
             rms = self.measure_rms(x)
-            change_dB = leveldBSPL - (equiv_0dB_SPL + 20 * torch.log10(rms + EPS))
-            x = x * 10 ** (0.05 * change_dB)
+            change_db = level_db_spl - (EQUIV_0_DB_SPL + 20 * torch.log10(rms + EPS))
+            x = x * 10 ** (0.05 * change_db)
         return x
 
-    def src_to_cochlea_filt(self, x, coch_filter):
-        return F.conv1d(x, coch_filter, padding=self.coch_padding)
+    def src_to_cochlea_filt(
+        self, x: torch.Tensor, cochlea_filter: torch.Tensor
+    ) -> torch.Tensor:
+        return F.conv1d(x, cochlea_filter, padding=self.cochlea_padding)
 
-    def smear(self, x):
+    def smear(self, x: torch.Tensor) -> torch.Tensor:
         """Padding issue needs to be worked out"""
         length = x.shape[2]
         x = x.view(x.shape[0], x.shape[2])
@@ -489,7 +391,7 @@ class MSBGHearingModel(nn.Module):
         phasor = spec[:, : self.smear_nfft // 2, :, :] / (mag + EPS)
 
         smeared_power = (
-            torch.matmul(power.transpose(-1, -2), self.fsmear.transpose(0, 1))
+            torch.matmul(power.transpose(-1, -2), self.f_smear.transpose(0, 1))
             .transpose(-1, -2)
             .unsqueeze(-1)
             + EPS
@@ -512,11 +414,11 @@ class MSBGHearingModel(nn.Module):
         )
         return smeared_wav.unsqueeze(1)
 
-    def recruitment(self, x):
-        nsamps = x.shape[-1]
+    def recruitment(self, x: torch.Tensor) -> torch.Tensor:
+        n_samples = x.shape[-1]
         ixhp = 0
         outputs = []
-        for ixch in range(self.nchans):
+        for ixch in range(self.n_chans):
             # Gammaton filtering
             pass_n = torchaudio.functional.lfilter(
                 x, self.gtn_denoms[ixch, :], self.gtn_nums[ixch, :]
@@ -528,84 +430,90 @@ class MSBGHearingModel(nn.Module):
             dly = self.gtn_delays[ixch]
 
             pass_n_cali = torch.zeros_like(pass_n)
-            pass_n_cali[:, :, : nsamps - dly] = pass_n[:, :, dly:nsamps]
+            pass_n_cali[:, :, : n_samples - dly] = pass_n[:, :, dly:n_samples]
             # Tail control
-            if ixch >= self.start2polehp:
+            if ixch >= self.start_2_pole_hp:
                 ixhp += 1
                 pass_n_cali = torchaudio.functional.lfilter(
                     pass_n_cali, self.hp_denoms[ixhp - 1, :], self.hp_nums[ixhp - 1, :]
                 )
 
             # Get the envelope
-            envlp_out = torchaudio.functional.lfilter(
-                torch.abs(pass_n_cali), self.chan_lpfA[ixch, :], self.chan_lpfB[ixch, :]
+            envelope_out = torchaudio.functional.lfilter(
+                torch.abs(pass_n_cali),
+                self.chan_lpf_a[ixch, :],
+                self.chan_lpf_b[ixch, :],
             )
-            envlp_out = torch.flip(envlp_out, dims=[-1])
-            envlp_out = torchaudio.functional.lfilter(
-                envlp_out, self.chan_lpfA[ixch, :], self.chan_lpfB[ixch, :]
+            envelope_out = torch.flip(envelope_out, dims=[-1])
+            envelope_out = torchaudio.functional.lfilter(
+                envelope_out, self.chan_lpf_a[ixch, :], self.chan_lpf_b[ixch, :]
             )
-            envlp_out = torch.flip(envlp_out, dims=[-1])
+            envelope_out = torch.flip(envelope_out, dims=[-1])
 
-            envlp_out = torch.clamp(envlp_out, min=EPS, max=self.envlp_max[ixch])
-            gain = (envlp_out / self.envlp_max[ixch]) ** self.expnsn_m1[ixch]
+            envelope_out = torch.clamp(
+                envelope_out, min=EPS, max=self.envelope_max[ixch]
+            )
+            gain = (envelope_out / self.envelope_max[ixch]) ** self.expansion_m1[ixch]
             outputs.append(gain * pass_n_cali)
 
         y = torch.stack(outputs, dim=-1).sum(dim=-1)
-        y = y * self.recruitmnet_out_coef
+        y = y * self.recruitment_out_coef
         return y
 
-    def recruitment_fir(self, x):
-        nsamps = x.shape[-1]
-        x = x.repeat([1, self.nchans, 1])
+    def recruitment_fir(self, x: torch.Tensor) -> torch.Tensor:
+        n_samples = x.shape[-1]
+        x = x.repeat([1, self.n_chans, 1])
         real = F.conv1d(
-            x, self.gt_fir, bias=None, padding=self.padding, groups=self.nchans
+            x, self.gt_fir, bias=None, padding=self.padding, groups=self.n_chans
         )
         imag = F.conv1d(
-            x, self.gt_fir_sin, bias=None, padding=self.padding, groups=self.nchans
+            x, self.gt_fir_sin, bias=None, padding=self.padding, groups=self.n_chans
         )
         real_cali = torch.zeros_like(real)
         imag_cali = torch.zeros_like(imag)
-        for i in range(self.nchans):
-            real_cali[:, i, : nsamps - self.peaks[i]] = real[
-                :, i, self.peaks[i] : nsamps
+        for i in range(self.n_chans):
+            real_cali[:, i, : n_samples - self.peaks[i]] = real[
+                :, i, self.peaks[i] : n_samples
             ]
-            imag_cali[:, i, : nsamps - self.peaks[i]] = imag[
-                :, i, self.peaks[i] : nsamps
+            imag_cali[:, i, : n_samples - self.peaks[i]] = imag[
+                :, i, self.peaks[i] : n_samples
             ]
 
         env = torch.sqrt(real_cali * real_cali + imag_cali * imag_cali + EPS)
         env = F.conv1d(
-            env, self.fir_lpf, bias=None, padding=self.padding, groups=self.nchans
+            env, self.fir_lpf, bias=None, padding=self.padding, groups=self.n_chans
         )
 
-        env_max = self.envlp_max.unsqueeze(0).unsqueeze(-1).repeat([1, 1, nsamps])
+        env_max = self.envelope_max.unsqueeze(0).unsqueeze(-1).repeat([1, 1, n_samples])
         gain = torch.clamp(env / env_max, min=EPS, max=1)
-        gain = gain ** self.expnsn_m1.unsqueeze(0).unsqueeze(-1).repeat([1, 1, nsamps])
+        gain = gain ** self.expansion_m1.unsqueeze(0).unsqueeze(-1).repeat(
+            [1, 1, n_samples]
+        )
         y = torch.sum(gain * real_cali, dim=1, keepdim=True)
-        y = y * self.recruitmnet_out_coef
+        y = y * self.recruitment_out_coef
         return y
 
-    def forward(self, x):
+    def forward(self, x: torch.Tensor) -> torch.Tensor:
         x = self.calibrate_spl(x)
         x = x.unsqueeze(1)
-        x = self.src_to_cochlea_filt(x, self.coch_filter_forward)
+        x = self.src_to_cochlea_filt(x, self.cochlea_filter_forward)
         x = self.smear(x)
         # x = self.recruitment(x)
         x = self.recruitment_fir(x)
-        y = self.src_to_cochlea_filt(x, self.coch_filter_backward)
+        y = self.src_to_cochlea_filt(x, self.cochlea_filter_backward)
         return y.squeeze(1)
 
 
 class torchloudnorm(nn.Module):
     def __init__(
         self,
-        sr=44100,
-        norm_lufs=-36,
-        kernel_size=1025,
-        block_size=0.4,
-        overlap=0.75,
-        gamma_a=-70,
-    ):
+        sr: int = 44100,
+        norm_lufs: int = -36,
+        kernel_size: int = 1025,
+        block_size: float = 0.4,
+        overlap: float = 0.75,
+        gamma_a: int = -70,
+    ) -> None:
         super().__init__()
         self.sr = sr
         self.norm_lufs = norm_lufs
@@ -647,12 +555,12 @@ class torchloudnorm(nn.Module):
         )
         self.gamma_a = gamma_a
 
-    def apply_filter(self, x):
+    def apply_filter(self, x: torch.Tensor) -> torch.Tensor:
         x = F.conv1d(x, self.high_shelf, padding=self.padding)
         x = F.conv1d(x, self.high_pass, padding=self.padding)
         return x
 
-    def integrated_loudness(self, x):
+    def integrated_loudness(self, x: torch.Tensor) -> torch.Tensor:
         x = self.apply_filter(x)
         x_unfold = self.unfold(x.unsqueeze(2))
 
@@ -675,12 +583,12 @@ class torchloudnorm(nn.Module):
         lufs = -0.691 + 10 * torch.log10(z_ave_gated_a_r + EPS)  # loudness
         return lufs
 
-    def normalize_loudness(self, x, lufs):
+    def normalize_loudness(self, x: torch.Tensor, lufs: torch.Tensor) -> torch.Tensor:
         delta_loudness = self.norm_lufs - lufs
         gain = torch.pow(10, delta_loudness / 20)
         return gain * x
 
-    def forward(self, x):
+    def forward(self, x: torch.Tensor) -> torch.Tensor:
         loudness = self.integrated_loudness(x.unsqueeze(1))
         y = self.normalize_loudness(x, loudness)
         return y
