@@ -40,7 +40,7 @@ class S2SBaseSearcher(torch.nn.Module):
     """
 
     def __init__(self, bos_index, eos_index, min_decode_ratio, max_decode_ratio):
-        super(S2SBaseSearcher, self).__init__()
+        super().__init__()
         self.bos_index = bos_index
         self.eos_index = eos_index
         self.min_decode_ratio = min_decode_ratio
@@ -59,7 +59,7 @@ class S2SBaseSearcher(torch.nn.Module):
         """
         raise NotImplementedError
 
-    def forward_step(self, inp_tokens, memory, enc_states, enc_lens):
+    def forward_step(self, inp_tokens, memory, enc_states, enc_lens, index=0):
         """This method should implement one step of
         forwarding operation in the autoregressive model.
 
@@ -74,6 +74,8 @@ class S2SBaseSearcher(torch.nn.Module):
             The encoder states to be attended.
         enc_lens : torch.Tensor
             The actual length of each enc_states sequence.
+        index : int
+            The current timestep index.
 
         Returns
         -------
@@ -153,6 +155,9 @@ class S2SGreedySearcher(S2SBaseSearcher):
     greedy decoding approach. See also S2SBaseSearcher().
     """
 
+    # pylint: disable=W0223
+    # This class is intentionally abstract so disable these warnings
+
     def forward(self, enc_states, wav_len):
         enc_lens = torch.round(enc_states.shape[1] * wav_len).int()
         device = enc_states.device
@@ -166,7 +171,7 @@ class S2SGreedySearcher(S2SBaseSearcher):
         log_probs_lst = []
         max_decode_steps = int(enc_states.shape[1] * self.max_decode_ratio)
 
-        for t in range(max_decode_steps):
+        for _t in range(max_decode_steps):
             log_probs, memory, _ = self.forward_step(
                 inp_tokens, memory, enc_states, enc_lens
             )
@@ -219,8 +224,11 @@ class S2SRNNGreedySearcher(S2SGreedySearcher):
     >>> hyps, scores = searcher(enc, wav_len)
     """
 
+    # pylint: disable=W0223
+    # Doesn't implement lm_forward_step() or reset_lm_mem()
+
     def __init__(self, embedding, decoder, linear, **kwargs):
-        super(S2SRNNGreedySearcher, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         self.emb = embedding
         self.dec = decoder
         self.fc = linear
@@ -235,7 +243,8 @@ class S2SRNNGreedySearcher(S2SGreedySearcher):
         c = torch.zeros(batch_size, self.dec.attn_dim, device=device)
         return hs, c
 
-    def forward_step(self, inp_tokens, memory, enc_states, enc_lens):
+    def forward_step(self, inp_tokens, memory, enc_states, enc_lens, _index=0):
+        # Note, _index is not used here, but it is defined in base class method
         hs, c = memory
         e = self.emb(inp_tokens)
         dec_out, hs, c, w = self.dec.forward_step(e, hs, c, enc_states, enc_lens)
@@ -276,7 +285,8 @@ class S2SBeamSearcher(S2SBaseSearcher):
     coverage_penalty: float
         The coefficient of coverage penalty (η).
         log P(y|x) + λ log P_LM(y) + γ*len(y) + η*coverage(x,y). (default: 0.0)
-        Reference: https://arxiv.org/pdf/1612.02695.pdf, https://arxiv.org/pdf/1808.10792.pdf
+        Reference: https://arxiv.org/pdf/1612.02695.pdf,
+            https://arxiv.org/pdf/1808.10792.pdf
     lm_weight : float
         The weight of LM when performing beam search (λ).
         log P(y|x) + λ log P_LM(y). (default: 0.0)
@@ -290,8 +300,8 @@ class S2SBeamSearcher(S2SBaseSearcher):
         CTC prefix scoring on "partial" token or "full: token.
     ctc_window_size: int
         Default: 0
-        Compute the ctc scores over the time frames using windowing based on attention peaks.
-        If 0, no windowing applied.
+        Compute the ctc scores over the time frames using windowing based on
+        attention peaks. If 0, no windowing applied.
     using_max_attn_shift: bool
         Whether using the max_attn_shift constraint. (default: False)
     max_attn_shift: int
@@ -329,10 +339,9 @@ class S2SBeamSearcher(S2SBaseSearcher):
         max_attn_shift=60,
         minus_inf=-1e20,
     ):
-        super(S2SBeamSearcher, self).__init__(
-            bos_index, eos_index, min_decode_ratio, max_decode_ratio
-        )
+        super().__init__(bos_index, eos_index, min_decode_ratio, max_decode_ratio)
         self.beam_size = beam_size
+        self.beam_offset = None
         self.topk = topk
         self.return_log_probs = return_log_probs
         self.length_normalization = length_normalization
@@ -365,7 +374,8 @@ class S2SBeamSearcher(S2SBaseSearcher):
         if self.ctc_weight > 0.0:
             if len({self.bos_index, self.eos_index, self.blank_index}) < 3:
                 raise ValueError(
-                    "To perform joint ATT/CTC decoding, set blank, eos and bos to different indexes."
+                    "To perform joint ATT/CTC decoding, set blank, eos and bos to "
+                    " different indexes."
                 )
 
         # ctc already initialized
@@ -380,7 +390,8 @@ class S2SBeamSearcher(S2SBaseSearcher):
         ---------
         hyps : List
             This list contains batch_size number.
-            Each inside list contains a list stores all the hypothesis for this sentence.
+            Each inside list contains a list storing all the hypothesis for
+                this sentence.
         beam_size : int
             The number of beam_size.
 
@@ -390,11 +401,8 @@ class S2SBeamSearcher(S2SBaseSearcher):
             Whether the hyps has been full.
         """
         hyps_len = [len(lst) for lst in hyps]
-        beam_size = [self.beam_size for _ in range(len(hyps_len))]
-        if hyps_len == beam_size:
-            return True
-        else:
-            return False
+        beam_size = [self.beam_size] * len(hyps_len)
+        return hyps_len == beam_size
 
     def _check_attn_shift(self, attn, prev_attn_peak):
         """This method checks whether attention shift is more than attn_shift.
@@ -490,15 +498,17 @@ class S2SBeamSearcher(S2SBaseSearcher):
                 hyps_and_scores[batch_id].append((hyp, log_probs, final_scores))
         return is_eos
 
-    def _get_top_score_prediction(self, hyps_and_scores, topk):
-        """This method sorts the scores and return corresponding hypothesis and log probs.
+    # pylint: disable=W0613
+    def _get_top_score_prediction(self, all_hyps_and_scores, topk):
+        """Sort the scores and return corresponding hypothesis and log probs.
 
         Arguments
         ---------
-        hyps_and_scores : list
+        all_hyps_and_scores : list
             To store generated hypotheses and scores.
         topk : int
             Number of hypothesis to return.
+            Deprecated, use self.topk instead.
 
         Returns
         -------
@@ -512,11 +522,11 @@ class S2SBeamSearcher(S2SBaseSearcher):
             The log probabilities of each hypotheses.
         """
         top_hyps, top_log_probs, top_scores, top_lengths = [], [], [], []
-        batch_size = len(hyps_and_scores)
+        batch_size = len(all_hyps_and_scores)
 
         # Collect hypotheses
-        for i in range(len(hyps_and_scores)):
-            hyps, log_probs, scores = zip(*hyps_and_scores[i])
+        for hyps_and_scores in all_hyps_and_scores:
+            hyps, log_probs, scores = zip(*hyps_and_scores)
             top_hyps += hyps
             top_scores += scores
             top_log_probs += log_probs
@@ -540,8 +550,9 @@ class S2SBeamSearcher(S2SBaseSearcher):
 
         return topk_hyps, topk_scores, topk_lengths, topk_log_probs
 
-    # def forward(self, enc_states, wav_len):  # noqa: C901
+    # Extra parameters have been added to overriden function
     def forward(self, src, tokens_bos, wav_lens, pad_idx):  # noqa: C901
+        # pylint: disable=arguments-differ
         all_enc_states = []
         all_enc_lens = []
         all_memory = []
@@ -609,7 +620,7 @@ class S2SBeamSearcher(S2SBaseSearcher):
         # keep only the first to make sure no redundancy.
         sequence_scores.index_fill_(0, self.beam_offset, 0.0)
 
-        # keep the hypothesis that reaches eos and their corresponding score and log_probs.
+        # keep the hypothesis that reaches eos and corresponding score and log_probs.
         hyps_and_scores = [[] for _ in range(batch_size)]
 
         # keep the sequences that still not reaches eos.
@@ -724,7 +735,7 @@ class S2SBeamSearcher(S2SBaseSearcher):
             if self.length_normalization:
                 sequence_scores = sequence_scores * (t + 1)
 
-            # The index of which beam the current top-K output came from in (t-1) timesteps.
+            # Index of which beam the current top-K output came from in (t-1) timesteps.
             predecessors = (
                 torch.div(candidates, vocab_size, rounding_mode="floor")
                 + self.beam_offset.unsqueeze(1).expand_as(candidates)
@@ -741,7 +752,7 @@ class S2SBeamSearcher(S2SBaseSearcher):
                         all_ctc_memory[j], candidates
                     )
 
-            # If using_max_attn_shift, then the previous attn peak has to be permuted too.
+            # If using_max_attn_shift, the previous attn peak has to be permuted too.
             if self.using_max_attn_shift:
                 prev_attn_peak = torch.index_select(
                     prev_attn_peak, dim=0, index=predecessors
@@ -756,9 +767,10 @@ class S2SBeamSearcher(S2SBaseSearcher):
                     # Init coverage
                     self.coverage = cur_attn
 
-                # the attn of transformer is [batch_size*beam_size, current_step, source_len]
+                # the attn of transformer is
+                #   [batch_size*beam_size, current_step, source_len]
                 if len(cur_attn.size()) > 2:
-                    self.converage = torch.sum(cur_attn, dim=1)
+                    self.coverage = torch.sum(cur_attn, dim=1)
                 else:
                     # Update coverage
                     self.coverage = torch.index_select(
@@ -843,8 +855,8 @@ class S2SBeamSearcher(S2SBaseSearcher):
                 # for entropy prediction
                 beam_logprobs = log_probs[j * self.topk : (j + 1) * self.topk]
                 beam_seq_probs = []
-                for m in range(len(beam_logprobs)):
-                    seq_prob = torch.sum(beam_logprobs[m])
+                for beam_logprob in beam_logprobs:
+                    seq_prob = torch.sum(beam_logprob)
                     beam_seq_probs.append(seq_prob)
                 beam_seq_probs = torch.stack(beam_seq_probs, dim=0)
                 beam_seq_probs_softmax = torch.nn.functional.softmax(beam_seq_probs)
@@ -862,8 +874,8 @@ class S2SBeamSearcher(S2SBaseSearcher):
                 uncertainty.append({"confidence": confidence, "entropy": entropy})
 
             return predictions, topk_scores, uncertainty
-        else:
-            return predictions, topk_scores
+
+        return predictions, topk_scores
 
     def ctc_forward_step(self, x, model_idx):
         logits = self.ctc_fc[model_idx](x)
@@ -951,10 +963,13 @@ class S2SRNNBeamSearcher(S2SBeamSearcher):
     >>> hyps, scores = searcher(enc, wav_len)
     """
 
+    # pylint: disable=W0223
+    # Doesn't implement lm_forward_step(), permute_lm_mem() or reset_lm_mem()
+
     def __init__(
         self, embedding, decoder, linear, ctc_linear=None, temperature=1.0, **kwargs
     ):
-        super(S2SRNNBeamSearcher, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         self.emb = embedding
         self.dec = decoder
         self.fc = linear
@@ -971,7 +986,8 @@ class S2SRNNBeamSearcher(S2SBeamSearcher):
         c = torch.zeros(batch_size, self.dec.attn_dim, device=device)
         return hs, c
 
-    def forward_step(self, inp_tokens, memory, enc_states, enc_lens):
+    def forward_step(self, inp_tokens, memory, enc_states, enc_lens, _index=0):
+        # Note, _index is not used here, but it is defined in base class method
         with torch.no_grad():
             hs, c = memory
             e = self.emb(inp_tokens)
@@ -1052,7 +1068,7 @@ class S2SRNNBeamSearchLM(S2SRNNBeamSearcher):
     def __init__(
         self, embedding, decoder, linear, language_model, temperature_lm=1.0, **kwargs
     ):
-        super(S2SRNNBeamSearchLM, self).__init__(embedding, decoder, linear, **kwargs)
+        super().__init__(embedding, decoder, linear, **kwargs)
 
         self.lm = language_model
         self.lm.eval()
@@ -1138,9 +1154,7 @@ class S2SRNNBeamSearchTransformerLM(S2SRNNBeamSearcher):
     def __init__(
         self, embedding, decoder, linear, language_model, temperature_lm=1.0, **kwargs
     ):
-        super(S2SRNNBeamSearchTransformerLM, self).__init__(
-            embedding, decoder, linear, **kwargs
-        )
+        super().__init__(embedding, decoder, linear, **kwargs)
 
         self.lm = language_model
         self.lm.eval()
@@ -1196,7 +1210,7 @@ def inflate_tensor(tensor, times, dim):
 
 
 def mask_by_condition(tensor, cond, fill_value):
-    """This function will mask some element in the tensor with fill_value, if condition=False.
+    """Mask some element in the tensor with fill_value, if condition=False.
 
     Arguments
     ---------
@@ -1262,7 +1276,7 @@ class S2STransformerBeamSearch(S2SBeamSearcher):
     """
 
     def __init__(self, modules, temperature=1.0, temperature_lm=1.0, **kwargs):
-        super(S2STransformerBeamSearch, self).__init__(**kwargs)
+        super().__init__(**kwargs)
 
         # self.model = modules[0]
         # self.fc = modules[1]
@@ -1293,10 +1307,10 @@ class S2STransformerBeamSearch(S2SBeamSearcher):
         memory = torch.index_select(memory, dim=0, index=index)
         return memory
 
-    def forward_step(self, inp_tokens, memory, enc_states, enc_lens, model_idx):
+    def forward_step(self, inp_tokens, memory, enc_states, enc_lens, index=0):
         memory = _update_mem(inp_tokens, memory)
-        pred, attn = self.model[model_idx].decode(memory, enc_states)
-        prob_dist = self.softmax(self.fc[model_idx](pred) / self.temperature)
+        pred, attn = self.model[index].decode(memory, enc_states)
+        prob_dist = self.softmax(self.fc[index](pred) / self.temperature)
         return prob_dist[:, -1, :], memory, attn
 
     def lm_forward_step(self, inp_tokens, memory):
